@@ -18,15 +18,13 @@ from .base import BaseEstimator, TransformerMixin, ClassifierMixin
 from .linear_model._base import LinearClassifierMixin
 from .covariance import ledoit_wolf, empirical_covariance, shrunk_covariance
 from .utils.multiclass import unique_labels
-from .utils import check_array
 from .utils.validation import check_is_fitted
 from .utils.multiclass import check_classification_targets
 from .utils.extmath import softmax
 from .preprocessing import StandardScaler
-from .utils.validation import _deprecate_positional_args
 
 
-__all__ = ['LinearDiscriminantAnalysis', 'QuadraticDiscriminantAnalysis']
+__all__ = ["LinearDiscriminantAnalysis", "QuadraticDiscriminantAnalysis"]
 
 
 def _cov(X, shrinkage=None, covariance_estimator=None):
@@ -63,30 +61,34 @@ def _cov(X, shrinkage=None, covariance_estimator=None):
     if covariance_estimator is None:
         shrinkage = "empirical" if shrinkage is None else shrinkage
         if isinstance(shrinkage, str):
-            if shrinkage == 'auto':
+            if shrinkage == "auto":
                 sc = StandardScaler()  # standardize features
                 X = sc.fit_transform(X)
                 s = ledoit_wolf(X)[0]
                 # rescale
                 s = sc.scale_[:, np.newaxis] * s * sc.scale_[np.newaxis, :]
-            elif shrinkage == 'empirical':
+            elif shrinkage == "empirical":
                 s = empirical_covariance(X)
             else:
-                raise ValueError('unknown shrinkage parameter')
+                raise ValueError("unknown shrinkage parameter")
         elif isinstance(shrinkage, float) or isinstance(shrinkage, int):
             if shrinkage < 0 or shrinkage > 1:
-                raise ValueError('shrinkage parameter must be between 0 and 1')
+                raise ValueError("shrinkage parameter must be between 0 and 1")
             s = shrunk_covariance(empirical_covariance(X), shrinkage)
         else:
-            raise TypeError('shrinkage must be a float or a string')
+            raise TypeError("shrinkage must be a float or a string")
     else:
         if shrinkage is not None and shrinkage != 0:
-            raise ValueError("covariance_estimator and shrinkage parameters "
-                             "are not None. Only one of the two can be set.")
+            raise ValueError(
+                "covariance_estimator and shrinkage parameters "
+                "are not None. Only one of the two can be set."
+            )
         covariance_estimator.fit(X)
-        if not hasattr(covariance_estimator, 'covariance_'):
-            raise ValueError("%s does not have a covariance_ attribute" %
-                             covariance_estimator.__class__.__name__)
+        if not hasattr(covariance_estimator, "covariance_"):
+            raise ValueError(
+                "%s does not have a covariance_ attribute"
+                % covariance_estimator.__class__.__name__
+            )
         s = covariance_estimator.covariance_
     return s
 
@@ -158,15 +160,14 @@ def _class_cov(X, y, priors, shrinkage=None, covariance_estimator=None):
     cov = np.zeros(shape=(X.shape[1], X.shape[1]))
     for idx, group in enumerate(classes):
         Xg = X[y == group, :]
-        cov += priors[idx] * np.atleast_2d(
-            _cov(Xg, shrinkage, covariance_estimator))
+        cov += priors[idx] * np.atleast_2d(_cov(Xg, shrinkage, covariance_estimator))
     return cov
 
 
-class LinearDiscriminantAnalysis(LinearClassifierMixin,
-                                 TransformerMixin,
-                                 BaseEstimator):
-    """Linear Discriminant Analysis
+class LinearDiscriminantAnalysis(
+    LinearClassifierMixin, TransformerMixin, BaseEstimator
+):
+    """Linear Discriminant Analysis.
 
     A classifier with a linear decision boundary, generated by fitting class
     conditional densities to the data and using Bayes' rule.
@@ -215,7 +216,7 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         `transform` method.
 
     store_covariance : bool, default=False
-        If True, explicitely compute the weighted within-class covariance
+        If True, explicitly compute the weighted within-class covariance
         matrix when solver is 'svd'. The matrix is always computed
         and stored for the other solvers.
 
@@ -280,6 +281,17 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
     classes_ : array-like of shape (n_classes,)
         Unique class labels.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     See Also
     --------
     QuadraticDiscriminantAnalysis : Quadratic Discriminant Analysis.
@@ -297,9 +309,16 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
     [1]
     """
 
-    def __init__(self, solver='svd', shrinkage=None, priors=None,
-                 n_components=None, store_covariance=False, tol=1e-4,
-                 covariance_estimator=None):
+    def __init__(
+        self,
+        solver="svd",
+        shrinkage=None,
+        priors=None,
+        n_components=None,
+        store_covariance=False,
+        tol=1e-4,
+        covariance_estimator=None,
+    ):
         self.solver = solver
         self.shrinkage = shrinkage
         self.priors = priors
@@ -356,14 +375,15 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
            0-471-05669-3.
         """
         self.means_ = _class_means(X, y)
-        self.covariance_ = _class_cov(X, y, self.priors_, shrinkage,
-                                      covariance_estimator)
+        self.covariance_ = _class_cov(
+            X, y, self.priors_, shrinkage, covariance_estimator
+        )
         self.coef_ = linalg.lstsq(self.covariance_, self.means_.T)[0].T
-        self.intercept_ = (-0.5 * np.diag(np.dot(self.means_, self.coef_.T)) +
-                           np.log(self.priors_))
+        self.intercept_ = -0.5 * np.diag(np.dot(self.means_, self.coef_.T)) + np.log(
+            self.priors_
+        )
 
-    def _solve_eigen(self, X, y, shrinkage,
-                     covariance_estimator):
+    def _solve_eigen(self, X, y, shrinkage, covariance_estimator):
         """Eigenvalue solver.
 
         The eigenvalue solver computes the optimal solution of the Rayleigh
@@ -409,22 +429,25 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
            0-471-05669-3.
         """
         self.means_ = _class_means(X, y)
-        self.covariance_ = _class_cov(X, y, self.priors_, shrinkage,
-                                      covariance_estimator)
+        self.covariance_ = _class_cov(
+            X, y, self.priors_, shrinkage, covariance_estimator
+        )
 
         Sw = self.covariance_  # within scatter
         St = _cov(X, shrinkage, covariance_estimator)  # total scatter
         Sb = St - Sw  # between scatter
 
         evals, evecs = linalg.eigh(Sb, Sw)
-        self.explained_variance_ratio_ = np.sort(evals / np.sum(evals)
-                                                 )[::-1][:self._max_components]
+        self.explained_variance_ratio_ = np.sort(evals / np.sum(evals))[::-1][
+            : self._max_components
+        ]
         evecs = evecs[:, np.argsort(evals)[::-1]]  # sort eigenvectors
 
         self.scalings_ = evecs
         self.coef_ = np.dot(self.means_, evecs).dot(evecs.T)
-        self.intercept_ = (-0.5 * np.diag(np.dot(self.means_, self.coef_.T)) +
-                           np.log(self.priors_))
+        self.intercept_ = -0.5 * np.diag(np.dot(self.means_, self.coef_.T)) + np.log(
+            self.priors_
+        )
 
     def _solve_svd(self, X, y):
         """SVD solver.
@@ -456,8 +479,8 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         # 1) within (univariate) scaling by with classes std-dev
         std = Xc.std(axis=0)
         # avoid division by zero in normalization
-        std[std == 0] = 1.
-        fac = 1. / (n_samples - n_classes)
+        std[std == 0] = 1.0
+        fac = 1.0 / (n_samples - n_classes)
 
         # 2) Within variance scaling
         X = np.sqrt(fac) * (Xc / std)
@@ -470,26 +493,34 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
 
         # 3) Between variance scaling
         # Scale weighted centers
-        X = np.dot(((np.sqrt((n_samples * self.priors_) * fac)) *
-                    (self.means_ - self.xbar_).T).T, scalings)
+        X = np.dot(
+            (
+                (np.sqrt((n_samples * self.priors_) * fac))
+                * (self.means_ - self.xbar_).T
+            ).T,
+            scalings,
+        )
         # Centers are living in a space with n_classes-1 dim (maximum)
         # Use SVD to find projection in the space spanned by the
         # (n_classes) centers
         _, S, Vt = linalg.svd(X, full_matrices=0)
 
-        self.explained_variance_ratio_ = (S**2 / np.sum(
-            S**2))[:self._max_components]
+        if self._max_components == 0:
+            self.explained_variance_ratio_ = np.empty((0,), dtype=S.dtype)
+        else:
+            self.explained_variance_ratio_ = (S ** 2 / np.sum(S ** 2))[
+                : self._max_components
+            ]
+
         rank = np.sum(S > self.tol * S[0])
         self.scalings_ = np.dot(scalings, Vt.T[:, :rank])
         coef = np.dot(self.means_ - self.xbar_, self.scalings_)
-        self.intercept_ = (-0.5 * np.sum(coef ** 2, axis=1) +
-                           np.log(self.priors_))
+        self.intercept_ = -0.5 * np.sum(coef ** 2, axis=1) + np.log(self.priors_)
         self.coef_ = np.dot(coef, self.scalings_.T)
         self.intercept_ -= np.dot(self.xbar_, self.coef_.T)
 
     def fit(self, X, y):
-        """Fit LinearDiscriminantAnalysis model according to the given
-           training data and parameters.
+        """Fit the Linear Discriminant Analysis model.
 
            .. versionchanged:: 0.19
               *store_covariance* has been moved to main constructor.
@@ -504,16 +535,23 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
 
         y : array-like of shape (n_samples,)
             Target values.
+
+        Returns
+        -------
+        self : object
+            Fitted estimator.
         """
-        X, y = self._validate_data(X, y, ensure_min_samples=2, estimator=self,
-                                   dtype=[np.float64, np.float32])
+        X, y = self._validate_data(
+            X, y, ensure_min_samples=2, estimator=self, dtype=[np.float64, np.float32]
+        )
         self.classes_ = unique_labels(y)
         n_samples, _ = X.shape
         n_classes = len(self.classes_)
 
         if n_samples == n_classes:
-            raise ValueError("The number of samples must be more "
-                             "than the number of classes.")
+            raise ValueError(
+                "The number of samples must be more than the number of classes."
+            )
 
         if self.priors is None:  # estimate priors from sample
             _, y_t = np.unique(y, return_inverse=True)  # non-negative ints
@@ -524,8 +562,7 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         if (self.priors_ < 0).any():
             raise ValueError("priors must be non-negative")
         if not np.isclose(self.priors_.sum(), 1.0):
-            warnings.warn("The priors do not sum to 1. Renormalizing",
-                          UserWarning)
+            warnings.warn("The priors do not sum to 1. Renormalizing", UserWarning)
             self.priors_ = self.priors_ / self.priors_.sum()
 
         # Maximum number of components no matter what n_components is
@@ -537,35 +574,46 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         else:
             if self.n_components > max_components:
                 raise ValueError(
-                    "n_components cannot be larger than min(n_features, "
-                    "n_classes - 1)."
+                    "n_components cannot be larger than min(n_features, n_classes - 1)."
                 )
             self._max_components = self.n_components
 
-        if self.solver == 'svd':
+        if self.solver == "svd":
             if self.shrinkage is not None:
-                raise NotImplementedError('shrinkage not supported')
+                raise NotImplementedError("shrinkage not supported")
             if self.covariance_estimator is not None:
                 raise ValueError(
-                        'covariance estimator '
-                        'is not supported '
-                        'with svd solver. Try another solver')
+                    "covariance estimator "
+                    "is not supported "
+                    "with svd solver. Try another solver"
+                )
             self._solve_svd(X, y)
-        elif self.solver == 'lsqr':
-            self._solve_lsqr(X, y, shrinkage=self.shrinkage,
-                             covariance_estimator=self.covariance_estimator)
-        elif self.solver == 'eigen':
-            self._solve_eigen(X, y,
-                              shrinkage=self.shrinkage,
-                              covariance_estimator=self.covariance_estimator)
+        elif self.solver == "lsqr":
+            self._solve_lsqr(
+                X,
+                y,
+                shrinkage=self.shrinkage,
+                covariance_estimator=self.covariance_estimator,
+            )
+        elif self.solver == "eigen":
+            self._solve_eigen(
+                X,
+                y,
+                shrinkage=self.shrinkage,
+                covariance_estimator=self.covariance_estimator,
+            )
         else:
-            raise ValueError("unknown solver {} (valid solvers are 'svd', "
-                             "'lsqr', and 'eigen').".format(self.solver))
+            raise ValueError(
+                "unknown solver {} (valid solvers are 'svd', "
+                "'lsqr', and 'eigen').".format(self.solver)
+            )
         if self.classes_.size == 2:  # treat binary case as a special case
-            self.coef_ = np.array(self.coef_[1, :] - self.coef_[0, :], ndmin=2,
-                                  dtype=X.dtype)
-            self.intercept_ = np.array(self.intercept_[1] - self.intercept_[0],
-                                       ndmin=1, dtype=X.dtype)
+            self.coef_ = np.array(
+                self.coef_[1, :] - self.coef_[0, :], ndmin=2, dtype=X.dtype
+            )
+            self.intercept_ = np.array(
+                self.intercept_[1] - self.intercept_[0], ndmin=1, dtype=X.dtype
+            )
         return self
 
     def transform(self, X):
@@ -581,18 +629,19 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         X_new : ndarray of shape (n_samples, n_components)
             Transformed data.
         """
-        if self.solver == 'lsqr':
-            raise NotImplementedError("transform not implemented for 'lsqr' "
-                                      "solver (use 'svd' or 'eigen').")
+        if self.solver == "lsqr":
+            raise NotImplementedError(
+                "transform not implemented for 'lsqr' solver (use 'svd' or 'eigen')."
+            )
         check_is_fitted(self)
 
-        X = check_array(X)
-        if self.solver == 'svd':
+        X = self._validate_data(X, reset=False)
+        if self.solver == "svd":
             X_new = np.dot(X - self.xbar_, self.scalings_)
-        elif self.solver == 'eigen':
+        elif self.solver == "eigen":
             X_new = np.dot(X, self.scalings_)
 
-        return X_new[:, :self._max_components]
+        return X_new[:, : self._max_components]
 
     def predict_proba(self, X):
         """Estimate probability.
@@ -612,7 +661,7 @@ class LinearDiscriminantAnalysis(LinearClassifierMixin,
         decision = self.decision_function(X)
         if self.classes_.size == 2:
             proba = expit(decision)
-            return np.vstack([1-proba, proba]).T
+            return np.vstack([1 - proba, proba]).T
         else:
             return softmax(decision)
 
@@ -683,7 +732,7 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
         where S2 corresponds to the `scaling_` attribute of a given class.
 
     store_covariance : bool, default=False
-        If True, the class covariance matrices are explicitely computed and
+        If True, the class covariance matrices are explicitly computed and
         stored in the `self.covariance_` attribute.
 
         .. versionadded:: 0.17
@@ -730,6 +779,17 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
     classes_ : ndarray of shape (n_classes,)
         Unique class labels.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     Examples
     --------
     >>> from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis
@@ -746,9 +806,10 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
     --------
     LinearDiscriminantAnalysis : Linear Discriminant Analysis.
     """
-    @_deprecate_positional_args
-    def __init__(self, *, priors=None, reg_param=0., store_covariance=False,
-                 tol=1.0e-4):
+
+    def __init__(
+        self, *, priors=None, reg_param=0.0, store_covariance=False, tol=1.0e-4
+    ):
         self.priors = np.asarray(priors) if priors is not None else None
         self.reg_param = reg_param
         self.store_covariance = store_covariance
@@ -767,8 +828,8 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Training vector, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vector, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         y : array-like of shape (n_samples,)
             Target values (integers)
@@ -779,8 +840,10 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
         n_samples, n_features = X.shape
         n_classes = len(self.classes_)
         if n_classes < 2:
-            raise ValueError('The number of classes has to be greater than'
-                             ' one; got %d class' % (n_classes))
+            raise ValueError(
+                "The number of classes has to be greater than one; got %d class"
+                % (n_classes)
+            )
         if self.priors is None:
             self.priors_ = np.bincount(y) / float(n_samples)
         else:
@@ -798,8 +861,10 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
             meang = Xg.mean(0)
             means.append(meang)
             if len(Xg) == 1:
-                raise ValueError('y has only 1 sample in class %s, covariance '
-                                 'is ill defined.' % str(self.classes_[ind]))
+                raise ValueError(
+                    "y has only 1 sample in class %s, covariance is ill defined."
+                    % str(self.classes_[ind])
+                )
             Xgc = Xg - meang
             # Xgc = U * S * V.T
             _, S, Vt = np.linalg.svd(Xgc, full_matrices=False)
@@ -824,7 +889,7 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
         # return log posterior, see eq (4.12) p. 110 of the ESL.
         check_is_fitted(self)
 
-        X = check_array(X)
+        X = self._validate_data(X, reset=False)
         norm2 = []
         for i in range(len(self.classes_)):
             R = self.rotations_[i]
@@ -834,7 +899,7 @@ class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
             norm2.append(np.sum(X2 ** 2, axis=1))
         norm2 = np.array(norm2).T  # shape = [len(X), n_classes]
         u = np.asarray([np.sum(np.log(s)) for s in self.scalings_])
-        return (-0.5 * (norm2 + u) + np.log(self.priors_))
+        return -0.5 * (norm2 + u) + np.log(self.priors_)
 
     def decision_function(self, X):
         """Apply decision function to an array of samples.
