@@ -7,12 +7,13 @@ from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 
 import numpy as np
-from joblib import Parallel, delayed
+from joblib import Parallel
 import scipy.sparse as sparse
 
 from ..base import clone
 from ..base import ClassifierMixin, RegressorMixin, TransformerMixin
 from ..base import is_classifier, is_regressor
+from ..exceptions import NotFittedError
 from ..utils._estimator_html_repr import _VisualBlock
 
 from ._base import _fit_single_estimator
@@ -31,17 +32,24 @@ from ..utils.metaestimators import if_delegate_has_method
 from ..utils.multiclass import check_classification_targets
 from ..utils.validation import check_is_fitted
 from ..utils.validation import column_or_1d
-from ..utils.validation import _deprecate_positional_args
+from ..utils.fixes import delayed
 
 
-class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
-                    metaclass=ABCMeta):
+class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble, metaclass=ABCMeta):
     """Base class for stacking method."""
 
     @abstractmethod
-    def __init__(self, estimators, final_estimator=None, *, cv=None,
-                 stack_method='auto', n_jobs=None, verbose=0,
-                 passthrough=False):
+    def __init__(
+        self,
+        estimators,
+        final_estimator=None,
+        *,
+        cv=None,
+        stack_method="auto",
+        n_jobs=None,
+        verbose=0,
+        passthrough=False,
+    ):
         super().__init__(estimators=estimators)
         self.final_estimator = final_estimator
         self.cv = cv
@@ -75,8 +83,10 @@ class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
             if preds.ndim == 1:
                 X_meta.append(preds.reshape(-1, 1))
             else:
-                if (self.stack_method_[est_idx] == 'predict_proba' and
-                        len(self.classes_) == 2):
+                if (
+                    self.stack_method_[est_idx] == "predict_proba"
+                    and len(self.classes_) == 2
+                ):
                     # Remove the first column when using probabilities in
                     # binary classification because both features are perfectly
                     # collinear.
@@ -92,19 +102,22 @@ class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
 
     @staticmethod
     def _method_name(name, estimator, method):
-        if estimator == 'drop':
+        if estimator == "drop":
             return None
-        if method == 'auto':
-            if getattr(estimator, 'predict_proba', None):
-                return 'predict_proba'
-            elif getattr(estimator, 'decision_function', None):
-                return 'decision_function'
+        if method == "auto":
+            if getattr(estimator, "predict_proba", None):
+                return "predict_proba"
+            elif getattr(estimator, "decision_function", None):
+                return "decision_function"
             else:
-                return 'predict'
+                return "predict"
         else:
             if not hasattr(estimator, method):
-                raise ValueError('Underlying estimator {} does not implement '
-                                 'the method {}.'.format(name, method))
+                raise ValueError(
+                    "Underlying estimator {} does not implement the method {}.".format(
+                        name, method
+                    )
+                )
             return method
 
     def fit(self, X, y, sample_weight=None):
@@ -144,19 +157,21 @@ class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
         # predict_proba. They are exposed publicly.
         self.estimators_ = Parallel(n_jobs=self.n_jobs)(
             delayed(_fit_single_estimator)(clone(est), X, y, sample_weight)
-            for est in all_estimators if est != 'drop'
+            for est in all_estimators
+            if est != "drop"
         )
-        self.n_features_in_ = self.estimators_[0].n_features_in_
 
         self.named_estimators_ = Bunch()
         est_fitted_idx = 0
         for name_est, org_est in zip(names, all_estimators):
-            if org_est != 'drop':
-                self.named_estimators_[name_est] = self.estimators_[
-                    est_fitted_idx]
+            if org_est != "drop":
+                current_estimator = self.estimators_[est_fitted_idx]
+                self.named_estimators_[name_est] = current_estimator
                 est_fitted_idx += 1
+                if hasattr(current_estimator, "feature_names_in_"):
+                    self.feature_names_in_ = current_estimator.feature_names_in_
             else:
-                self.named_estimators_[name_est] = 'drop'
+                self.named_estimators_[name_est] = "drop"
 
         # To train the meta-classifier using the most data as possible, we use
         # a cross-validation to obtain the output of the stacked estimators.
@@ -165,57 +180,76 @@ class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
         # need to set the random state of the cv if there is one and we need to
         # take a copy.
         cv = check_cv(self.cv, y=y, classifier=is_classifier(self))
-        if hasattr(cv, 'random_state') and cv.random_state is None:
+        if hasattr(cv, "random_state") and cv.random_state is None:
             cv.random_state = np.random.RandomState()
 
         self.stack_method_ = [
             self._method_name(name, est, meth)
             for name, est, meth in zip(names, all_estimators, stack_method)
         ]
-        fit_params = ({"sample_weight": sample_weight}
-                      if sample_weight is not None
-                      else None)
+        fit_params = (
+            {"sample_weight": sample_weight} if sample_weight is not None else None
+        )
         predictions = Parallel(n_jobs=self.n_jobs)(
-            delayed(cross_val_predict)(clone(est), X, y, cv=deepcopy(cv),
-                                       method=meth, n_jobs=self.n_jobs,
-                                       fit_params=fit_params,
-                                       verbose=self.verbose)
+            delayed(cross_val_predict)(
+                clone(est),
+                X,
+                y,
+                cv=deepcopy(cv),
+                method=meth,
+                n_jobs=self.n_jobs,
+                fit_params=fit_params,
+                verbose=self.verbose,
+            )
             for est, meth in zip(all_estimators, self.stack_method_)
-            if est != 'drop'
+            if est != "drop"
         )
 
         # Only not None or not 'drop' estimators will be used in transform.
         # Remove the None from the method as well.
         self.stack_method_ = [
-            meth for (meth, est) in zip(self.stack_method_, all_estimators)
-            if est != 'drop'
+            meth
+            for (meth, est) in zip(self.stack_method_, all_estimators)
+            if est != "drop"
         ]
 
         X_meta = self._concatenate_predictions(X, predictions)
-        _fit_single_estimator(self.final_estimator_, X_meta, y,
-                              sample_weight=sample_weight)
+        _fit_single_estimator(
+            self.final_estimator_, X_meta, y, sample_weight=sample_weight
+        )
 
         return self
 
+    @property
+    def n_features_in_(self):
+        """Number of features seen during :term:`fit`."""
+        try:
+            check_is_fitted(self)
+        except NotFittedError as nfe:
+            raise AttributeError(
+                f"{self.__class__.__name__} object has no attribute n_features_in_"
+            ) from nfe
+        return self.estimators_[0].n_features_in_
+
     def _transform(self, X):
         """Concatenate and return the predictions of the estimators."""
         check_is_fitted(self)
         predictions = [
             getattr(est, meth)(X)
             for est, meth in zip(self.estimators_, self.stack_method_)
-            if est != 'drop'
+            if est != "drop"
         ]
         return self._concatenate_predictions(X, predictions)
 
-    @if_delegate_has_method(delegate='final_estimator_')
+    @if_delegate_has_method(delegate="final_estimator_")
     def predict(self, X, **predict_params):
         """Predict target for X.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Training vectors, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vectors, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         **predict_params : dict of str -> obj
             Parameters to the `predict` called by the `final_estimator`. Note
@@ -230,17 +264,18 @@ class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble,
         """
 
         check_is_fitted(self)
-        return self.final_estimator_.predict(
-            self.transform(X), **predict_params
-        )
+        return self.final_estimator_.predict(self.transform(X), **predict_params)
 
     def _sk_visual_block_(self, final_estimator):
         names, estimators = zip(*self.estimators)
-        parallel = _VisualBlock('parallel', estimators, names=names,
-                                dash_wrapped=False)
-        serial = _VisualBlock('serial', (parallel, final_estimator),
-                              dash_wrapped=False)
-        return _VisualBlock('serial', [serial])
+        parallel = _VisualBlock("parallel", estimators, names=names, dash_wrapped=False)
+
+        # final estimator is wrapped in a parallel block to show the label:
+        # 'final_estimator' in the html repr
+        final_block = _VisualBlock(
+            "parallel", [final_estimator], names=["final_estimator"], dash_wrapped=False
+        )
+        return _VisualBlock("serial", (parallel, final_block), dash_wrapped=False)
 
 
 class StackingClassifier(ClassifierMixin, _BaseStacking):
@@ -255,10 +290,10 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
     is trained using cross-validated predictions of the base estimators using
     `cross_val_predict`.
 
-    .. versionadded:: 0.22
-
     Read more in the :ref:`User Guide <stacking>`.
 
+    .. versionadded:: 0.22
+
     Parameters
     ----------
     estimators : list of (str, estimator)
@@ -268,7 +303,8 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
 
     final_estimator : estimator, default=None
         A classifier which will be used to combine the base estimators.
-        The default classifier is a `LogisticRegression`.
+        The default classifier is a
+        :class:`~sklearn.linear_model.LogisticRegression`.
 
     cv : int, cross-validation generator or an iterable, default=None
         Determines the cross-validation splitting strategy used in
@@ -281,8 +317,11 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         * An iterable yielding train, test splits.
 
         For integer/None inputs, if the estimator is a classifier and y is
-        either binary or multiclass, `StratifiedKFold` is used. In all other
-        cases, `KFold` is used.
+        either binary or multiclass,
+        :class:`~sklearn.model_selection.StratifiedKFold` is used.
+        In all other cases, :class:`~sklearn.model_selection.KFold` is used.
+        These splitters are instantiated with `shuffle=False` so the splits
+        will be the same across calls.
 
         Refer :ref:`User Guide <cross_validation>` for the various
         cross-validation strategies that can be used here.
@@ -331,6 +370,17 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
     named_estimators_ : :class:`~sklearn.utils.Bunch`
         Attribute to access any fitted sub-estimators by name.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying classifier exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Only defined if the
+        underlying estimators expose such an attribute when fit.
+        .. versionadded:: 1.0
+
     final_estimator_ : estimator
         The classifier which predicts given the output of `estimators_`.
 
@@ -376,10 +426,18 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
     0.9...
 
     """
-    @_deprecate_positional_args
-    def __init__(self, estimators, final_estimator=None, *, cv=None,
-                 stack_method='auto', n_jobs=None, passthrough=False,
-                 verbose=0):
+
+    def __init__(
+        self,
+        estimators,
+        final_estimator=None,
+        *,
+        cv=None,
+        stack_method="auto",
+        n_jobs=None,
+        passthrough=False,
+        verbose=0,
+    ):
         super().__init__(
             estimators=estimators,
             final_estimator=final_estimator,
@@ -387,15 +445,16 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
             stack_method=stack_method,
             n_jobs=n_jobs,
             passthrough=passthrough,
-            verbose=verbose
+            verbose=verbose,
         )
 
     def _validate_final_estimator(self):
         self._clone_final_estimator(default=LogisticRegression())
         if not is_classifier(self.final_estimator_):
             raise ValueError(
-                "'final_estimator' parameter should be a classifier. Got {}"
-                .format(self.final_estimator_)
+                "'final_estimator' parameter should be a classifier. Got {}".format(
+                    self.final_estimator_
+                )
             )
 
     def fit(self, X, y, sample_weight=None):
@@ -424,15 +483,15 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         self.classes_ = self._le.classes_
         return super().fit(X, self._le.transform(y), sample_weight)
 
-    @if_delegate_has_method(delegate='final_estimator_')
+    @if_delegate_has_method(delegate="final_estimator_")
     def predict(self, X, **predict_params):
         """Predict target for X.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Training vectors, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vectors, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         **predict_params : dict of str -> obj
             Parameters to the `predict` called by the `final_estimator`. Note
@@ -448,7 +507,7 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         y_pred = super().predict(X, **predict_params)
         return self._le.inverse_transform(y_pred)
 
-    @if_delegate_has_method(delegate='final_estimator_')
+    @if_delegate_has_method(delegate="final_estimator_")
     def predict_proba(self, X):
         """Predict class probabilities for X using
         `final_estimator_.predict_proba`.
@@ -456,8 +515,8 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Training vectors, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vectors, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         Returns
         -------
@@ -468,7 +527,7 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         check_is_fitted(self)
         return self.final_estimator_.predict_proba(self.transform(X))
 
-    @if_delegate_has_method(delegate='final_estimator_')
+    @if_delegate_has_method(delegate="final_estimator_")
     def decision_function(self, X):
         """Predict decision function for samples in X using
         `final_estimator_.decision_function`.
@@ -476,8 +535,8 @@ class StackingClassifier(ClassifierMixin, _BaseStacking):
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Training vectors, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vectors, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         Returns
         -------
@@ -527,10 +586,10 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
     is trained using cross-validated predictions of the base estimators using
     `cross_val_predict`.
 
-    .. versionadded:: 0.22
-
     Read more in the :ref:`User Guide <stacking>`.
 
+    .. versionadded:: 0.22
+
     Parameters
     ----------
     estimators : list of (str, estimator)
@@ -540,7 +599,7 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
 
     final_estimator : estimator, default=None
         A regressor which will be used to combine the base estimators.
-        The default regressor is a `RidgeCV`.
+        The default regressor is a :class:`~sklearn.linear_model.RidgeCV`.
 
     cv : int, cross-validation generator or an iterable, default=None
         Determines the cross-validation splitting strategy used in
@@ -553,8 +612,11 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
         * An iterable yielding train, test splits.
 
         For integer/None inputs, if the estimator is a classifier and y is
-        either binary or multiclass, `StratifiedKFold` is used. In all other
-        cases, `KFold` is used.
+        either binary or multiclass,
+        :class:`~sklearn.model_selection.StratifiedKFold` is used.
+        In all other cases, :class:`~sklearn.model_selection.KFold` is used.
+        These splitters are instantiated with `shuffle=False` so the splits
+        will be the same across calls.
 
         Refer :ref:`User Guide <cross_validation>` for the various
         cross-validation strategies that can be used here.
@@ -589,10 +651,23 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
     named_estimators_ : :class:`~sklearn.utils.Bunch`
         Attribute to access any fitted sub-estimators by name.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying regressor exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Only defined if the
+        underlying estimators expose such an attribute when fit.
+        .. versionadded:: 1.0
 
     final_estimator_ : estimator
         The regressor to stacked the base estimators fitted.
 
+    stack_method_ : list of str
+        The method used by each base estimator.
+
     References
     ----------
     .. [1] Wolpert, David H. "Stacked generalization." Neural networks 5.2
@@ -623,9 +698,17 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
     0.3...
 
     """
-    @_deprecate_positional_args
-    def __init__(self, estimators, final_estimator=None, *, cv=None,
-                 n_jobs=None, passthrough=False, verbose=0):
+
+    def __init__(
+        self,
+        estimators,
+        final_estimator=None,
+        *,
+        cv=None,
+        n_jobs=None,
+        passthrough=False,
+        verbose=0,
+    ):
         super().__init__(
             estimators=estimators,
             final_estimator=final_estimator,
@@ -633,15 +716,16 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
             stack_method="predict",
             n_jobs=n_jobs,
             passthrough=passthrough,
-            verbose=verbose
+            verbose=verbose,
         )
 
     def _validate_final_estimator(self):
         self._clone_final_estimator(default=RidgeCV())
         if not is_regressor(self.final_estimator_):
             raise ValueError(
-                "'final_estimator' parameter should be a regressor. Got {}"
-                .format(self.final_estimator_)
+                "'final_estimator' parameter should be a regressor. Got {}".format(
+                    self.final_estimator_
+                )
             )
 
     def fit(self, X, y, sample_weight=None):
@@ -650,8 +734,8 @@ class StackingRegressor(RegressorMixin, _BaseStacking):
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Training vectors, where n_samples is the number of samples and
-            n_features is the number of features.
+            Training vectors, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
 
         y : array-like of shape (n_samples,)
             Target values.
